@@ -1,6 +1,6 @@
 /*
  * OLSR ad-hoc routing table management protocol GUI front-end
- * Copyright (C) 2003 Andreas Tnnesen (andreto@ifi.uio.no)
+ * Copyright (C) 2003 Andreas Tønnesen (andreto@ifi.uio.no)
  *
  * This file is part of olsr.org.
  *
@@ -21,7 +21,7 @@
  */
 
 /*
- *Andreas Tnnesen (andreto@ifi.uio.no)
+ *Andreas Tønnesen (andreto@ifi.uio.no)
  *
  *The list of cached packets
  *
