@@ -54,8 +54,8 @@ struct ipv4_route {
 void init_zebra (void);
 void zebra_cleanup (void);
 unsigned char zebra_send_command (unsigned char, unsigned char *, int );
-int zebra_add_v4_route (struct ipv4_route r);
-int zebra_delete_v4_route (struct ipv4_route r);
+int zebra_add_v4_route (const struct ipv4_route r);
+int zebra_delete_v4_route (const struct ipv4_route r);
 void zebra_check (void*);
 int zebra_parse_packet (unsigned char*, ssize_t);
 int zebra_redistribute (unsigned char);
@@ -63,8 +63,8 @@ int zebra_disable_redistribute (unsigned char);
 int add_hna4_route (struct ipv4_route);
 int delete_hna4_route (struct ipv4_route);
 void *my_realloc (void *, size_t, const char *);
-int zebra_add_olsr_v4_route (struct rt_entry *);
-int zebra_del_olsr_v4_route (struct rt_entry *);
+int zebra_add_olsr_v4_route (const struct rt_entry *);
+int zebra_del_olsr_v4_route (const struct rt_entry *);
 void zebra_olsr_localpref (void);
 void zebra_olsr_distance (unsigned char);
 void zebra_export_routes(unsigned char);
