@@ -37,7 +37,6 @@
  * to the project. For more information see the website or contact
  * the copyright holders.
  *
- * $Id: glua.c,v 1.4 2007/04/20 13:46:03 bernd67 Exp $
  */
 
 #include "lua/lua.h"
@@ -294,7 +293,7 @@ static int tasWrite(lua_State *lua)
   int i;
   struct connInfo *info;
 
-  lua_pushlightuserdata(lua, (void *)&infoKey);
+  lua_pushlightuserdata(lua, &infoKey);
   lua_gettable(lua, LUA_REGISTRYINDEX);
 
   info = lua_touserdata(lua, -1);
@@ -313,7 +312,7 @@ static int tasWrite(lua_State *lua)
     if (strConv == NULL)
       return luaL_error(lua, "cannot convert value to string");
 
-    writeBuff(&info->write[2], (unsigned char *)strConv, strlen(strConv));
+    writeBuff(&info->write[2], (const unsigned char *)strConv, strlen(strConv));
 
     lua_pop(lua, 1);
   }
@@ -326,7 +325,7 @@ static int tasAddHeaderLine(lua_State *lua)
   struct connInfo *info;
   char *line;
 
-  lua_pushlightuserdata(lua, (void *)&infoKey);
+  lua_pushlightuserdata(lua, &infoKey);
   lua_gettable(lua, LUA_REGISTRYINDEX);
 
   info = lua_touserdata(lua, -1);
@@ -335,8 +334,8 @@ static int tasAddHeaderLine(lua_State *lua)
 
   chomp(line, strlen(line));
 
-  writeBuff(&info->write[1], (unsigned char *)line, strlen(line));
-  writeBuff(&info->write[1], (unsigned char *)"\r\n", 2);
+  writeBuff(&info->write[1], (const unsigned char *)line, strlen(line));
+  writeBuff(&info->write[1], (const unsigned char *)"\r\n", 2);
 
   freeMem(line);
 
@@ -347,16 +346,17 @@ static int tasSetContentType(lua_State *lua)
 {
   struct connInfo *info;
   const char *contType;
+  char *s;
 
-  lua_pushlightuserdata(lua, (void *)&infoKey);
+  lua_pushlightuserdata(lua, &infoKey);
   lua_gettable(lua, LUA_REGISTRYINDEX);
 
   info = lua_touserdata(lua, -1);
 
   contType = luaL_checkstring(lua, 1);
 
-  info->contType = allocBuff(info, strlen(contType) + 1);
-  strcpy(info->contType, contType);
+  s = allocBuff(info, strlen(contType) + 1);
+  strcpy(s, contType);
 
   return 0;
 }
@@ -365,7 +365,7 @@ static int tasKeepState(lua_State *lua)
 {
   int *keepFlag;
 
-  lua_pushlightuserdata(lua, (void *)&keepFlagKey);
+  lua_pushlightuserdata(lua, &keepFlagKey);
   lua_gettable(lua, LUA_REGISTRYINDEX);
 
   keepFlag = lua_touserdata(lua, -1);
@@ -380,7 +380,7 @@ static int tasGetParameters(lua_State *lua)
   int i;
   char **argList;
 
-  lua_pushlightuserdata(lua, (void *)&argListKey);
+  lua_pushlightuserdata(lua, &argListKey);
   lua_gettable(lua, LUA_REGISTRYINDEX);
 
   argList = lua_touserdata(lua, -1);
@@ -447,16 +447,16 @@ int runLua(char **errMsg, struct connInfo *info, const char *workDir,
   else
     lua = *session;
 
-  lua_pushlightuserdata(lua, (void *)&infoKey);
-  lua_pushlightuserdata(lua, (void *)info);
+  lua_pushlightuserdata(lua, &infoKey);
+  lua_pushlightuserdata(lua, info);
   lua_settable(lua, LUA_REGISTRYINDEX);
     
-  lua_pushlightuserdata(lua, (void *)&argListKey);
-  lua_pushlightuserdata(lua, (void *)argList);
+  lua_pushlightuserdata(lua, &argListKey);
+  lua_pushlightuserdata(lua, argList);
   lua_settable(lua, LUA_REGISTRYINDEX);
     
-  lua_pushlightuserdata(lua, (void *)&keepFlagKey);
-  lua_pushlightuserdata(lua, (void *)&keepFlag);
+  lua_pushlightuserdata(lua, &keepFlagKey);
+  lua_pushlightuserdata(lua, &keepFlag);
   lua_settable(lua, LUA_REGISTRYINDEX);
 
   res = luaL_loadfile(lua, lexPath);
