@@ -36,7 +36,6 @@
  * to the project. For more information see the website or contact
  * the copyright holders.
  *
- * $Id: olsr_protocol.h,v 1.22 2007/04/22 20:25:48 bernd67 Exp $
  */
 
 /*
@@ -48,6 +47,9 @@
 #define	_PROTOCOLS_OLSR_H
 
 #include "olsr_types.h"
+#include "olsr_cfg.h"
+
+#include <string.h>
 
 /* Port for OLSR to use */
 
@@ -185,16 +187,6 @@
 #define EXTRACT_LINK(link_code) (link_code & 0x3)
 
 
-/*
- * Macros for comparing and copying IP addresses
- */
-
-#define COMP_IP(ip1, ip2) (!memcmp(ip1, ip2, olsr_cnf->ipsize))
-
-#define COPY_IP(to, from) memcpy(to, from, olsr_cnf->ipsize)
-
-
-
 /***********************************************
  *           OLSR packet definitions           *
  ***********************************************/
